<?php

namespace Bixo\Aml\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Bixo\Aml\Http\Requests\NameRequest;
use Bixo\Aml\Interfaces\NameRepositoryInterface;
use Bixo\Aml\Models\Name;

/**
 * Resource controller class for name.
 */
class NameResourceController extends BaseController
{

    /**
     * Initialize name resource controller.
     *
     * @param type NameRepositoryInterface $name
     *
     * @return null
     */
    public function __construct(NameRepositoryInterface $name)
    {
        parent::__construct();
        $this->repository = $name;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Aml\Repositories\Criteria\NameResourceCriteria::class);
    }

    /**
     * Display a list of name.
     *
     * @return Response
     */
    public function index(NameRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Bixo\Aml\Repositories\Presenter\NamePresenter::class)
                ->$function();
        }

        $names = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('aml::name.names'))
            ->view('aml::name.index', true)
            ->data(compact('names', 'view'))
            ->output();
    }

    /**
     * Display name.
     *
     * @param Request $request
     * @param Model   $name
     *
     * @return Response
     */
    public function show(NameRequest $request, Name $name)
    {

        if ($name->exists) {
            $view = 'aml::name.show';
        } else {
            $view = 'aml::name.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('aml::name.name'))
            ->data(compact('name'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new name.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(NameRequest $request)
    {

        $name = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('aml::name.name')) 
            ->view('aml::name.create', true) 
            ->data(compact('name'))
            ->output();
    }

    /**
     * Create new name.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(NameRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $name                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('aml::name.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('aml/name/' . $name->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/aml/name'))
                ->redirect();
        }

    }

    /**
     * Show name for editing.
     *
     * @param Request $request
     * @param Model   $name
     *
     * @return Response
     */
    public function edit(NameRequest $request, Name $name)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('aml::name.name'))
            ->view('aml::name.edit', true)
            ->data(compact('name'))
            ->output();
    }

    /**
     * Update the name.
     *
     * @param Request $request
     * @param Model   $name
     *
     * @return Response
     */
    public function update(NameRequest $request, Name $name)
    {
        try {
            $attributes = $request->all();

            $name->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('aml::name.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('aml/name/' . $name->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('aml/name/' . $name->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the name.
     *
     * @param Model   $name
     *
     * @return Response
     */
    public function destroy(NameRequest $request, Name $name)
    {
        try {

            $name->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('aml::name.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('aml/name/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('aml/name/' . $name->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple name.
     *
     * @param Model   $name
     *
     * @return Response
     */
    public function delete(NameRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('aml::name.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('aml/name'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/aml/name'))
                ->redirect();
        }

    }

    /**
     * Restore deleted names.
     *
     * @param Model   $name
     *
     * @return Response
     */
    public function restore(NameRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('aml::name.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/aml/name'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/aml/name/'))
                ->redirect();
        }

    }

}
